// Pingus - A free Lemmings clone
// Copyright (C) 1999 Ingo Ruhnke <grumbel@gmx.de>

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <cassert>
#include <ClanLib/clanlib.h>
#include <ClanLib/clanlayer2.h>

#include "pingu.hh"
#include "actions/pingu_actions.hh"

// Create the static objects of the class
ColMap*             Pingu::colmap;

CL_Surface*         Pingu::walker;
CL_Surface*         Pingu::faller;
CL_Surface*         Pingu::tumble;

CL_ResourceManager* Pingu::resource;

// Init a pingu at the given position while falling
Pingu::Pingu(int x, int y)
{
  action = 0;
  data.x_pos = x;
  data.y_pos = y;
  walker_c.set_size(walker->get_no_sprs() / 2);
  walker_c.set_speed(50);
  //  walker_c.set_size(3);
  faller_c.set_size(faller->get_no_sprs());
  tumble_c.set_size(tumble->get_no_sprs());
  data.falling = 4;
  data.direction.left();
}

// Returns the x position of the pingu
int
Pingu::get_x()
{
  return data.x_pos;
}

// Returns the y position of the pingu
int
Pingu::get_y()
{
  return data.y_pos;
}

// Set the position of the pingu
void 
Pingu::set_pos(int x, int y)
{
  data.x_pos = x;
  data.y_pos = y;
}

// Static function which set the colmap for all pingus
void
Pingu::set_colmap(ColMap* a)
{
  colmap = a;
}

// Static function which set the map for all pingus
void
Pingu::set_map(PinguMap* a)
{
  // not implemented
}

// Static function which load the default surfaces for all pingus
void 
Pingu::load_data(CL_ResourceManager* res)
{
  assert(res);
  resource = res;

  walker = CL_Res_Surface::load("Pingus/walker", *resource);
  faller = CL_Res_Surface::load("Pingus/faller", *resource);
  tumble = CL_Res_Surface::load("Pingus/tumble", *resource);

  assert(walker);
  assert(faller);
  assert(tumble);
}

// Set the action of the pingu (bridger, blocker, bomber, etc.)
int
Pingu::set_action(PinguAction* act)
{
  if (!act)
    return 0;

  if (action)
    delete action;

  action = act;
  action->set_pingu_data(data);
  action->load_data(resource);
  return 1;
}

PinguStatus
Pingu::get_status(void)
{
  return data.status;
}

// Returns true if the given koordinates are above the pingu
bool
Pingu::is_over(int x, int y)
{
  if (data.status == dead || data.status == not_catchable)
    return false;

  if (data.x_pos - 16 < x && data.x_pos + 16 > x &&
      data.y_pos - 32 < y && data.y_pos > y) {
    return true;
  } else {
    return false;
  }
}

// Let the pingu do his job (i.e. walk his way)
void
Pingu::let_move(void)
{
  assert(colmap);
  action->set_colmap(colmap); //BUG: the colmap need only to be set once
  
  
  if (action && action->is_finished) {
    delete action;
    action = 0;
  }

  if (action) { // if it has a action, do it
    action->let_move();
  } else {      // if not, do a walk
    if (rel_getpixel(0, -1) == 0) { // if pingu is not on ground
      ++(data.y_pos);
      if (rel_getpixel(0, -1) == 0) 
	++(data.y_pos);
      ++(data.falling);
    } else { // if pingu is on ground
      data.falling = 0;
      if (rel_getpixel(1, 0) == 0) { // if infront is free
	data.x_pos += data.direction;    
      } else { // if infront is a pixel 
	// Pingu is walking mounten up/down
	if (rel_getpixel(1,1)== 0) {
	  data.x_pos += data.direction;
	  --data.y_pos;
	} else if (rel_getpixel(1,2)== 0) {
	  data.x_pos += data.direction;
	  data.y_pos -= 2;
	} else {
	  data.direction.change();
	}
      }
    }
  }
}

// Draws the pingu on the screen with the given offset
void
Pingu::draw_offset(int x, int y)
{
  y += 2;
  
  if (action)
    action->draw_offset(x, y);
  else {
    if      (data.falling > 75)
      tumble->put_screen(data.x_pos + x - 16, data.y_pos + y - 32,
			 ++tumble_c);
    else if (data.falling > 3)
      faller->put_screen(data.x_pos + x - 16, data.y_pos + y - 32,
			 ++faller_c);
    else
      walker->put_screen(data.x_pos + x - 16, data.y_pos + y - 32, 
			 ++walker_c
			 + ((data.direction.is_left() ? 0 :
			     walker_c.size())));
  }
}

// Returns the `color' of the colmap in the walking direction 
int
Pingu::rel_getpixel(int x, int y)
{
  assert(colmap);
  return colmap->getpixel(data.x_pos + (x * data.direction), (data.y_pos) - y);
}

// Let the pingu catch another pingu, so that an action can be aplied
// (i.e. let a blocker change the direction f another pingu)
void
Pingu::catch_pingu(Pingu* pingu)
{
  action->catch_pingu(&(pingu->data));
}

// Returns true if the pingu needs to catch another pingu
bool
Pingu::need_catch()
{
  if (action)
    return action->need_catch();
  else 
    return false;
}

/* EOF */
