//    ___
//   |  _\ A free Lemmings clone
//   |   /_  _ _  ___  _   _  ___ 
//   |  || || \ || _ || |_| ||_ -'
//   |__||_||_\_||_  ||_____||___|
//                _| |
//               |___|
//
//   Copyright (C) 1998 Ingo Ruhnke <grumbel@gmx.de>
//     
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <iostream>
#include <ctime>
#include <cstdio>
#include <unistd.h>
#include <getopt.h>

#include <ClanLib/clanlib.h>
#include <ClanLib/clanlayer2.h>

#include "window.hh"
#include "playfield.hh"
#include "button_panel.hh"
#include "pingus.hh"
#include "pingu.hh"
#include "globals.hh"
#include "pingulevelmap.hh"
#include "actions/pingu_actions.hh"
#include "levelinterrupt.hh"
#include "SoundSpecs.hh"

char* 
PingusMain::get_title()
{
  static char* title;
  
  if (!title) {
    title = new char[1024];
    strcpy(title, "Pingus ");
    strcat(title, VERSION);
    strcat(title, " - A free Lemmings clone");
  }

  return title;
}

// check_ars() checks the command line for options and set the
// corresponding global variables to the set values.
void 
PingusMain::check_args(int argc, char* argv[])
{
  int c; 
  int option_index = 0;
  int display_help = 0;

  struct option long_options[] =
  {
    {"music",           no_argument,       0, 'm'},
    {"speed",           required_argument, 0, 't'},
    {"sound",           no_argument,       0, 's'},
    {"data-path",       required_argument, 0, 'p'},
    {"level",           required_argument, 0, 'l'},
    {"help",            no_argument,       0, 'h'},
    {"version",         no_argument,       0, 'V'},
    {"verbose",         no_argument,       0, 'v'},
    {"frame-rate",      required_argument, 0, 'f'},
    {"use-dirty-tiles", no_argument,       0, 'd'},
    {"print-fps",       no_argument,       0, 'b'},
    {"split-screen",    no_argument,       0, 'x'},
    {0, 0, 0, 0}
  };

  while(true) {
    c = getopt_long(argc, argv, "vds:l:hVf:p:bx", long_options, &option_index);
    
    if (c == -1)
      break;

    switch(c) {
    case 0:
      cout << "You should normaly never reach this line." << endl;
      exit(EXIT_FAILURE);
      break;
    case 'x': // -x, --split-screen
      split_screen = true;
      break;
    case 'b': // -b, --print-fps
      print_fps = true;
      break;
    case 'l': // -l, --level
      levelfile = optarg;
      break;
    case 't': // -t, --speed
      game_speed = atoi(optarg);
      break;
    case 's': // -s, --sound
      if (verbose) cout << "Sound Effects enabled" << endl;
      sound_enabled = true;
      break;
    case 'm': // -m, --music
      if (verbose) cout << "Music enabled" << endl;
      music_enabled = true;
      break;
    case 'f': // -f
      global_frame_rate = atoi(optarg);
      break;
    case 'd': // -d
      use_dirty_tiles = true;
      break;
    case 'p': // -p
      pingus_data_dir = optarg;
      break;
    case 'V':
      cout << PACKAGE << " " << VERSION << endl;
      exit(EXIT_SUCCESS);
      break;
    case 'v':
      ++verbose;
      cout << "Being more verbose: " << verbose << endl;
      break;
    default:
      cout << "Usage: " << argv[0] << " [OPTIONS]..." << endl;
      cout <<
	"\n"
	"Options:\n"
	"   -h, --help              Displays this screen\n"
	"   -s, --sound             Enable sound\n"
	"   -m, --music             Enable music\n"
       	"   -p, --data-path path    Set the path to load the data files to `path'\n"
	"   -l, --level FILE        Load a custom level from `file'\n"
	"   -f, --frame-rate INT    Sets the frame rate of the game\n"
	"   -d, --use-dirty-tiles   Activate the new dirty tile mechanism (outdated)\n"
	"   -t, --speed SPEED       Set the game speed (0=fastest, >0=slower)\n"
	"   -b, --print-fps         Prints the fps to stdout\n"
	"   -v, --verbose           Print some more messages to stdout\n"
	"   --split-screen          Splits the screen in two parts\n"
	"   -V, --version           Prints version number and exit\n"
	   << endl;
      exit(EXIT_SUCCESS);
      break;
    }
  }
}

void
PingusMain::intro(void)
{
  // Show a simple startup intro
}

// Loads a level from file and starts playing i
Result
PingusMain::play_level(const string& filename)
{
  if (verbose)
    cout << "Constructing Window Objs" << endl;

  Result result;
  PLF*         plf          = new PLF(filename.c_str());
  Window*      game         = new Window();

  // Create GuiObj for Window* game 
  Playfield*   playfield    = new Playfield(plf, result);
  ButtonPanel* button_panel = new ButtonPanel(plf);
  LevelMap*    level_map    = new LevelMap(plf);
  if (verbose)
    cout << "Constructing Window Objs finished" << endl;

  playfield->set_clip_rect(0, 0, 640, 420);

  // Connect the button_panel with the playfield
  playfield->set_buttons(button_panel);

  // Add all objects to the window
  game->add_obj(playfield);
  game->add_obj(button_panel);
  game->add_obj(level_map);

  // Give main controll to the window
  if (verbose)
    cout << "Entering main_loop. Startup time: " 
	 << CL_System::get_time() << " msec." << endl;

  // This loop will never exit, only throu a throw/catch pair
  game->main_loop();

  // Delete all alocated objects
  delete    level_map;
  delete button_panel;
  delete    playfield;
  delete         game;
  delete          plf;

  return result;
}

void
PingusMain::load_data()
{
  if (verbose) {
    cout << "Loading data... " << endl;
    cout << pingus_scriptfile <<  endl;
    cout << pingus_datafile << endl;
  }
  // The next line won't work, don't know why, but ClanLib says:
  // ClanLib: Couldn't find datafile index: /home/ingo/cvs/pingus/data/pingus.scr
  // resource = new CL_ResourceManager(pingus_scriptfile.c_str(), pingus_datafile.c_str());

  resource = new CL_ResourceManager(pingus_scriptfile.c_str());

  Pingu::load_data(resource);
  PinguAction::load_data(resource);
  World::load_data(resource);
  Playfield::load_data(resource);
  ButtonPanel::load_data(resource);
  LevelMap::load_data(resource);
  Entrance::load_data(resource);
  Trap::load_data(resource);
  Exit::load_data(resource);

  if (verbose)
    cout << "Loading data successfull" << endl;
}
  
void
PingusMain::init(int argc, char* argv[])
{
  // Print a nice ansi logo on startup (quite useless feature)
#ifdef NICE_STARTUP_LOGO
  cout << "[0;1;30;44m         [40m      [44m       [40m"
"[1;44m        [40m        [44m      [2C[47m  [5C      [1C  [3C  [1C  [2C  [1C  [2C  [40m"
"[1;30;44m        [40m [47m  [40m [47m  [40m  [44m      [2C[47m  [7C  [3C   [2C  [1C  [2C  [2C    [40m"
"[1;30;44m        [40m [47m [40m   [47m [40m   [44m     [2C[47m  [7C  [3C    [1C  [1C  [2C  [3C  [40m"
"[1;30;44m        [40m  [43m    [40m   [44m     [2C[47m  [7C  [3C  [1C    [1C  [2C  [2C    [40m"
"[1;30;44m        [40m [0;30;43m`----'[40m  [1;44m     [2C[47m      [1C      [1C  [3C  [2C    [2C  [2C  [40m"
"[1;30;44m        [40m [47m.[37m    [30m.[40m   [44m    [40m"
"[1;30;44m       [40m [47m        [40m    [44m  [40m[2C[1;37;40m"
"[1;30;44m       [40m [47m         [40m    [44m [40m[2C[1;37;40m[0m"
"[1;30;44m       [40m [47m         [40m    [30;44m [40m[2C[1;37;40m[0m"
"[1;30;44m      [40m [47m          [40m     [2C[1;37;40mPingus V.0.0.8[0m"
"[1;30;44m     [40m  [47m          [40m     [2C[1;37;40m[0m"
"[1;30;44m     [43m   [30;47m        [43m   [40m"
"[1;44m   [43m     [47m        [43m    [40m"
"[1;44m [43m       [47m        [43m      [40m"
"[1;44m [43m       [40m [47m      [40m [43m      [40m"
"[0m[255D" << endl;
#endif
  PingusMain::check_args(argc, argv);

  string hotspot_file;
  // Get the data directory from a enviroment variable or from a
  // preset value
  if (pingus_data_dir == "") {
    char* env = getenv("PINGUS_DATA_DIR");
    if (env)
      pingus_data_dir = env;
    else 
      pingus_data_dir = "/usr/local/share/pingus/";
  }
  
  pingus_datafile   = pingus_data_dir + "pingus.dat";
  pingus_scriptfile = pingus_data_dir + "pingus.scr";
  pingus_soundfile  = pingus_data_dir + "sounds/" + "sound.specs";
  hotspot_file      = pingus_data_dir + "hotspots.specs";

  if (verbose) {
    cout << "Pingus Data File:   " << pingus_datafile << endl;
    cout << "Pingus Scrpit File: " << pingus_scriptfile << endl;
    cout << "Hotspot Offset File"  << hotspot_file << endl;
    cout << "Pingus Sound File: "  << pingus_soundfile << endl;
  }

  cout << "Reading Sound Specs" << endl;
  SoundSpecs sound(pingus_soundfile);
  cout << "Reading Sound Specs finished" << endl;

  if (verbose) { cout << "Reading info" << endl; }
  Hotspot::read_info(hotspot_file.c_str());
  if (verbose) { cout << "finished" << endl; }
  
  if (levelfile == "") 
    levelfile = pingus_data_dir + "level3.plf";
    
  if (verbose)
    cout << "Pingus Level File: " << levelfile << endl;

  // initialisation ClanLib
  CL_System::init_display();

  if (sound_enabled || music_enabled)
    CL_System::init_sound();
  
  CL_Display::set_videomode(640, 480, 16);

  register_actions();
  load_data();
  srand(time(NULL));
  
  /*  if (!Mouse::init()) {
    cout << "No Mouse Support available" << endl;
    }*/
}

int
PingusMain::main(int argc, char** argv)
{
  if (verbose)
    cout << "Starting Main: " << CL_System::get_time() << endl;
  
  Result result;
  
  try {
    init(argc, argv);  
    intro();
    
  restart_level: // Shot me, I used a 'goto'
    try { result = play_level(levelfile); }
    
    catch (LevelInterrupt i) {
      switch (i.status) {
      case LevelInterrupt::restart:
	goto restart_level;
	break;
      case LevelInterrupt::quit:
	cout << "Going to quit" << endl;
	break;
      default:
	cout << "Undefined value for LevelInterrupt" << endl;
      }
    }

    cout << "Pingus exit normaly" << endl;
    result.print();
  }
  catch (CL_Error err)
  {
    cout << "Error caught from ClanLib: " << err.message << endl;
  }

  return 0;
}

/* EOF */
